#include <osg/Node>
#include <osg/Group>
#include <osg/Geode>
#include <osg/ShapeDrawable>
#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>

#include "TetrisBlock.h"

extern double tetrisblock_offsets[72];

TetrisBlock::TetrisBlock(int block_idx, osg::Vec3d& position, int numturnsx, int numturnsy, int numturnsz)
{
	set(block_idx, position, numturnsx, numturnsy, numturnsz);
}

void TetrisBlock::set(int block_idx, osg::Vec3d& position, int numturnsx, int numturnsy, int numturnsz)
{
	// TODO
	// You can set your colors here. Try a static array that defines colors for each
	// block like I did with tetrisblock_offsets, then find a way to change colors to
	// Geodes in OSG (setMaterial or a color NodeVisitor)
	osg::Geode* gblock = new osg::Geode();
	gblock->addDrawable(new osg::ShapeDrawable(new osg::Box()));

	osg::ref_ptr<osg::PositionAttitudeTransform> pat_alloc(new osg::PositionAttitudeTransform);
	pat = pat_alloc;

	osg::ref_ptr<osg::PositionAttitudeTransform> offsetpat0(new osg::PositionAttitudeTransform);
	offsetpat0->setPosition(osg::Vec3d(0,0,0)); // base
	offsetpat0->addChild(gblock);
	pat->addChild(offsetpat0);
	
	for(int i = 0; i < 3; ++i) {
		osg::ref_ptr<osg::PositionAttitudeTransform> offsetpat(new osg::PositionAttitudeTransform);
		offsetpat->setPosition(getTetrisAtom(block_idx, i)); // offset
		offsetpat->addChild(gblock);
		pat->addChild(offsetpat);
	}

	rotate(numturnsx, numturnsy, numturnsz);
	pat->setPosition(position);
}

void TetrisBlock::attachToGroup(osg::ref_ptr<osg::Group>& group)
{
	group->addChild(pat);
}

void TetrisBlock::rotate(int numturnsx, int numturnsy, int numturnsz)
{
	osg::Quat rotx(numturnsx*osg::PI_2, osg::Vec3d(1,0,0));
	osg::Quat roty(numturnsy*osg::PI_2, osg::Vec3d(0,1,0));
	osg::Quat rotz(numturnsz*osg::PI_2, osg::Vec3d(0,0,1));
	pat->setAttitude(rotx*roty*rotz);
}

void TetrisBlock::translate(osg::Vec3d& pos)
{
	pat->setPosition(pos);
}

osg::Vec3d TetrisBlock::getTetrisAtom(int block_id, int atom_id) const
{
	double xx = tetrisblock_offsets[9*block_id + 3*atom_id];
	double yy = tetrisblock_offsets[9*block_id + 3*atom_id + 1];
	double zz = tetrisblock_offsets[9*block_id + 3*atom_id + 2];
	return osg::Vec3d(xx,yy,zz);
}

// implement me
void TetrisBlock::getGlobalAtomPositions(AABB a[4]) const
{
	// TODO
	// compute the global atom positions using PositionAttitudeTransform pat
	// and each PAT for each atom and store them in a[0], a[1], a[2], a[3]
}

bool TetrisBlock::collidesWith(const TetrisBlock& tblock) const
{
	AABB a[4];
	AABB b[4];
	getGlobalAtomPositions(a);
	tblock.getGlobalAtomPositions(b);
	for(int i = 0; i < 4; ++i) {
		for(int j = 0; j < 4; ++j) {
			if(a[i].collidesWith(b[j])) return true;
		}
		// if above doesn't vectorize when compiled, try unrolling for speed
		// if your computer is slow. (the number of blocks used in this project shouldn't
		// make this necessary, but just in case...)
		//if(a[i].collidesWith(b[0])) return true;
		//if(a[i].collidesWith(b[1])) return true;
		//if(a[i].collidesWith(b[2])) return true;
		//if(a[i].collidesWith(b[3])) return true;
	}
	return false;
}

// implement me, too
bool AABB::collidesWith(const AABB& bb) const
{
	// TODO
	// collision detection for two axis-aligned bounding boxes
	// make sure the AABB's don't have overlapping ranges on each access
	// and return true if they do. If they don't overlap, then return false.
	return false;
}