#ifndef _TETRIS_PIECES_H
#define _TETRIS_PIECES_H

#include <osg/PositionAttitudeTransform>
#include <osg/Array>
#include <osg/Group>
#include <osg/Quat>

// all offsets are relative to base
static double tetrisblock_offsets[72] = {
	0,0,-1, // long piece
	0,0,1,
	0,0,2,
	0,0,-1, // T-piece
	0,1,0,
	0,0,1,
	0,1,0, // L-piece
	0,0,1,
	0,0,2,
	0,0,1, // zigzag piece
	0,1,0,
	0,1,-1,
	0,1,0, // cross-axis zigzag
	1,0,0,
	1,0,1,
	0,-1,0, // mirror cross-axis zigzag
	1,0,0,
	1,0,1,
	0,0,1, // tripod
	1,0,0,
	0,1,0,
	0,1,0, // square piece
	0,0,1,
	0,1,1
};

// Axis-Aligned Bounding Box
class AABB
{
public:
	AABB() { }
	~AABB() { }
	osg::Vec3d bmin;
	osg::Vec3d bmax;

	inline bool collidesWith(const AABB& bb) const;
};

class TetrisBlock
{
public:
	TetrisBlock() { }
	TetrisBlock(int block_idx, osg::Vec3d& position, int numturnsx, int numturnsy, int numturnsz);
	~TetrisBlock() { }
	void attachToGroup(osg::ref_ptr<osg::Group>& group);
	void rotate(int numturnsx, int numturnsy, int numturnsz);
	void translate(osg::Vec3d& pos);
	void set(int block_idx, osg::Vec3d& position, int numturnsx, int numturnsy, int numturnsz);
	inline bool collidesWith(const TetrisBlock& tblock) const;
	void getGlobalAtomPositions(AABB a[4]) const;

private:
	osg::ref_ptr<osg::PositionAttitudeTransform> pat;

	osg::Vec3d getTetrisAtom(int block_id, int atom_id) const;
};

#endif